//   Copyright (c)  2014  Anna M Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
//#include "CoCoA/assert.H"  // for using TEST_ASSERT
#include "CoCoA/error.H"
#include "CoCoA/RingElemInput.H"  // for ReadExpr

#include "CoCoA/library.H"


#include <iostream>
using std::cerr;
using std::endl;

#include <vector>
using std::vector;



//----------------------------------------------------------------------
// Test for implicitization
// functions: ImplicitDirect, elim, SliceCore
// environments: ZZ/(32003)[x,y,z,w], ZZ/(32003)[s,t,u]
//----------------------------------------------------------------------

// Handy macro for making assertions.
#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


namespace CoCoA
{

  void test(const vector<RingElem>& L, long sl, std::string algo)
  {
    double T0 = CpuTime();
    RingElem f;
    std::cout << sl << "sl + "<< algo << std::flush;
    try { f = SliceCore(L, sl, algo); }
    catch (const CoCoA::ErrorInfo& err) {cerr<<"***ERROR***"; ANNOUNCE(cerr,err);}
    std::cout << "\t time: " << CpuTime()-T0;
    std::cout << "\t ("<< NumTerms(f) << ")" << std::endl;
  }

  void program()
  {
    GlobalManager CoCoAFoundations;

    //  vector<symbol> syms = symbols("s", "t");
    //  vector<symbol> syms = symbols("s", "t", "u");
    vector<symbol> syms = symbols("s", "t", "u", "w");
    SparsePolyRing P = NewPolyRing(NewRingFp(32003), syms);
    vector<RingElem> L;

    std::cout << "*************************************************" << std::endl;
    // vector<RingElem> LA;
    // LA = CoCoAVector(ReadExpr(P, "s^7-s*t^3-t"),
    //                  ReadExpr(P, "s*t^3-s"),
    //                  ReadExpr(P, "s^13-t^2"));
    // // LA = CoCoAVector(ReadExpr(P, "u^2+s"),
    // // 		   ReadExpr(P, "s^2-s*t-t*u"),
    // // 		   ReadExpr(P, "s^3-t^2+u"),
    // // 		   ReadExpr(P, "s*t^2-s*u")
    // // 		   );
  

    std::cout << "*************************************************" << std::endl;
    L.clear();
    std::cout << " ES_9 = ";
    L.push_back(ReadExpr(P, "s^2-t^2+w"));
    L.push_back(ReadExpr(P, "s^2-u-w"));
    L.push_back(ReadExpr(P, "s^2-t*u"));
    L.push_back(ReadExpr(P, "t^2-s*u"));
    L.push_back(ReadExpr(P, "s^3+t-u-w"));

    // std::cout << " ES_8 = ";
    // L.push_back(ReadExpr(P, "s^3-t^2+u"));
    // L.push_back(ReadExpr(P, "s^2-t-w"));
    // L.push_back(ReadExpr(P, "s^2-t*u"));
    // L.push_back(ReadExpr(P, "s*t^2-s*u"));
    // L.push_back(ReadExpr(P, "s+t-u^2-w"));

    std::cout << L << std::endl;
    //  test(L, 1, "elim");
    // test(L, 3, "IDWCLPP2");
    test(L, 2, "IDWCLPP2");

  }

} // end of namespace CoCoA


//**********************************************************************
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}
